/* AP Radar
 * June 2002
 * donald.g.park@iname.com
 *
 * Released under the MIT Open Source Licence
 * http://www.opensource.org/licenses/mit-license.php
 */

#include <gtk--/button.h>
#include <gtk--/frame.h>
#include <gtk--/box.h>
#include <gtk--/window.h>
#include <gtk--/main.h>
#include <gtk--/pixmap.h>
#include <gtk--/base.h>
#include <gtk--/menu.h>
#include <gtk--/menushell.h>
#include <gtk--/label.h>
#include <stdio.h>                                // for sprintf
#include <iostream>
#include <sstream>
#include <string>

#include "Window.h"
#include "Parent.h"
#include "Main.h"
#include "../src/AccessPointList.h"
#include "../src/NetworkInterface.h"
#include "../src/Ipv4Address.h"

using SigC::slot;
using SigC::bind;

using namespace std;
using namespace Gtk::Menu_Helpers;

Window::Window()
:  Gtk::Window(GTK_WINDOW_TOPLEVEL)
{
}


void Window::init()
{

    // The idle call is in Window because I could only register a method
    // of a class that inherits from GTK. Only this class has the mojo,
    // register_data(), that makes the slot call work.
    // Connect idle function - 5000 milisecond delay
    Gtk::Main::timeout.connect(SigC::slot(this, &Window::goIdle), 5000);
    //Gtk::Main::idle.connect(SigC::slot(this, &Window::goIdle));

    Gtk::Frame *frame_horz;

    char title[128];
    sprintf(title, "AP Radar %s", VERSION);
    set_title (title);
    set_border_width (0);

    main_vbox = manage( new Gtk::VBox(false, 0) );
    add (*main_vbox);

    Gtk::Pixmap* logo;
    string logopath = (*(conf->getDirname()))+"/logo.xpm";
    //cout << " logo path " << logopath << endl;
    logo = manage( new Gtk::Pixmap (logopath.c_str()) );
    main_vbox->pack_start (*logo, true, true, 0);

    frame_horz = manage( new Gtk::Frame ("Access Point List") );
    main_vbox->pack_start (*frame_horz, true, true, 0);

    eth_horz = manage( new Gtk::Frame ("eth?") );
    main_vbox->pack_start (*eth_horz, true, true, 0);

    aplist_vbox = manage( new Gtk::VBox(false, 0) );
    frame_horz->add(*aplist_vbox);
    //main_vbox->pack_start(*button2, true, true, 10);

    eth_vbox = manage( new Gtk::VBox(false, 0) );
    eth_horz->add(*eth_vbox);

    //    button = manage ( new Gtk::Button ("Start scan"));
    //    button->clicked.connect(bind<char*>(slot(this, &Window::on_button_clicked),
    //        "doscan"));
    //    main_vbox->pack_start (*button, false, true, 0);

    show_all ();

    interface_essid_hbox = 0;

}


Window::~Window() {}

void Window::DHCPon()
{
    dhcp_vbox = manage( new Gtk::VBox(false, 0) );
    Gtk::Label* label = manage ( new Gtk::Label() );
    label->set("dhcpcd started");
    dhcp_vbox->pack_start(*label,true,true,0);
    //eth_horz->add(*dhcp_vbox);
    show_all();
}


void Window::DHCPoff()
{
}


gint Window::goIdle()
{
    cout << endl;
    cout << " == Timer started AP Scan == " << endl;
    //conf->syncAPList(conf->getMain()->ScanAPs());
    //setInterface(conf->getInterface());
    // DEBUG
    //NetworkInterface* nif = new NetworkInterface();
    //nif->setName(new string("eth0"));
    //conf->getInterface()->ping(new string("127.0.0.1"));
    return 1;
}


gint Window::delete_event_impl (GdkEventAny*)
{
    Gtk::Main::quit();
    return 0;
}


AccessPointList* ScanAPs();
void Window::on_button_clicked(char *data)
{
    cout << "button push - data:" << "\"" << data << "\"" << endl;
    //conf->syncAPList(conf->getMain()->ScanAPs());

}


void Window::AP_button_clicked(AccessPoint *data)
{
    cout << "an AP button was pushed - " << data->getEssid() << endl;
    // build a nemu - this should be on a right click. not sure how todo that.
    menu = manage(new Gtk::Menu());
    menu->items().push_back(MenuElem("Associate", AccelKey("<control>a")));
    menu->items().push_back(MenuElem("Monitor", AccelKey("<control>m")));
    //add popup stuff later
    //menu->popup(0,0);

    if(conf->getInterface() != NULL)
    {
        conf->getInterface()->Associate(data);
        DHCPon();
    }
    // make the window refresh
    setInterface(conf->getInterface());
}


void Window::add_AP(AccessPoint* ap)
{
    cout << "adding button " << *(ap->getEssid()) << endl;

    button = manage ( new Gtk::Button (ap->getEssid()->c_str()));
    button->set_relief(GTK_RELIEF_NONE);
    button->clicked.connect(bind<AccessPoint*>(slot(this, &Window::AP_button_clicked), ap));

    Gtk::Box *hbox;
    hbox = manage( new Gtk::HBox(false, 0) );
    //main_vbox->pack_start (*button, true, true, 10);
    aplist_vbox->pack_start (*hbox, true, true, 0);
    hbox->pack_start(*button, true, true, 0);
    string* modebuttonname;
    if(ap->getModeString() != NULL)
    {
        modebuttonname = ap->getModeString();
    }
    else
    {
        modebuttonname = new string("Unknown mode");
    }

    button = manage ( new Gtk::Button (modebuttonname->c_str()));
    button->set_relief(GTK_RELIEF_NONE);
    hbox->pack_start(*button, true, true, 0);

    Gtk::Pixmap* logo;
    string* wepbuttonname;
    if(ap->getWEP() == -1)
    {
        wepbuttonname = new string("WEP unknown");
    }
    if(ap->getWEP() == 0)
    {
        wepbuttonname = new string("wep off");
        string iconpath = (*(conf->getDirname()))+"/wepoff.xpm";
        logo = manage( new Gtk::Pixmap (iconpath ));
    }
    if(ap->getWEP() == 1)
    {
        wepbuttonname = new string("wep on");
        string iconpath = (*(conf->getDirname()))+"/wepon.xpm";
        logo = manage( new Gtk::Pixmap (iconpath ));
    }

    button = manage ( new Gtk::Button ());
    button->add(*logo);
    //button = manage ( new Gtk::Button (wepbuttonname->c_str()));
    button->set_relief(GTK_RELIEF_NONE);
    hbox->pack_start(*button, true, true, 0);

    string* frequencyname;
    char buf[15];
    //sprintf(buf,"%1.3fGHz", ap->getFrequency() / 1000 / 1000 / 1000);
    sprintf(buf,"Ch %d", ap->getChannel());
    frequencyname = new string(buf);
    button = manage ( new Gtk::Button (frequencyname->c_str()));
    button->set_relief(GTK_RELIEF_NONE);
    hbox->pack_start(*button, true, true, 0);

    string* signalstrengthname;
    //sprintf(buf,"%1.3fGHz", ap->getFrequency() / 1000 / 1000 / 1000);
    sprintf(buf,"%d dBm", ap->getSignalStrength());
    signalstrengthname = new string(buf);
    button = manage ( new Gtk::Button (signalstrengthname->c_str()));
    button->set_relief(GTK_RELIEF_NONE);
    hbox->pack_start(*button, true, true, 0);

    show_all();
}


void Window::setConfig(Config* c)
{
    conf = c;
}


void Window::setInterface(NetworkInterface* n)
{
    if (n == NULL) return;

    if(interface_essid_hbox != 0 )
    {
        eth_vbox->remove(*interface_essid_hbox);
    }

    const Gtk::nstring* str2;
    str2 = new Gtk::nstring(*(n->getName()));
    eth_horz->set_label(*str2);

    interface_essid_hbox = manage( new Gtk::HBox(false, 0) );
    //main_vbox->pack_start (*button, true, true, 10);
    eth_vbox->pack_start (*interface_essid_hbox, true, true, 0);

    button = manage ( new Gtk::Button (n->getEssid()->c_str()) );
    button->set_relief(GTK_RELIEF_NONE);
    interface_essid_hbox->pack_start(*button, true, true, 0);

    button = manage ( new Gtk::Button (n->getModeString()->c_str()) );
    button->set_relief(GTK_RELIEF_NONE);
    interface_essid_hbox->pack_start(*button, true, true, 0);

    string* ipaddr = new string;
    Ipv4Address* ipaddy = n->getIpv4Address();
    if(ipaddy != NULL)
    {
        // weird ass C++ shit. In java this would be i.o1+"."+i.o2...
        ostringstream o;
        o << ipaddy->getOctet(0);
        o << ".";
        o << ipaddy->getOctet(1);
        o << ".";
        o << ipaddy->getOctet(2);
        o << ".";
        o << ipaddy->getOctet(3);
        ipaddr = new string(o.str());
    }
    else
    {
        ipaddr = new string("__.__.__.__");
    }
    button = manage ( new Gtk::Button (ipaddr->c_str()) );
    button->set_relief(GTK_RELIEF_NONE);
    interface_essid_hbox->pack_start(*button, true, true, 0);

    show_all();
}


/*
 * Copyright (c) 2002 Donald G. Park
 * The MIT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
