/* AP Radar
 * June 2002
 * donald.g.park@iname.com
 *
 * Released under the MIT Open Source Licence
 * http://www.opensource.org/licenses/mit-license.php
 */

#include <iostream>
#include <fstream>
#include <sys/socket.h>
#include <sys/stat.h>
#include "Config.h"
#include "AccessPointList.h"

using namespace std;

Config::Config()
{
    dirname = new string("no directory set");
    interface = NULL;
    dhcpCommand = new string ("pump -i ");

    /* Create a channel to the NET kernel. */
    int skfd;
    if((skfd = socket(AF_INET, SOCK_DGRAM,0) ) < 0)
    {
        cout << "socket creation in Config::Config()" << endl;
    }
    setSkfd(skfd);
}


void Config::setMain(Main* m)
{
    main = m;
}


Main* Config::getMain()
{
    return main;
}


void Config::setSkfd(int s)
{
    skfd = s;
}


int Config::getSkfd()
{
    return skfd;
}


void Config::setInterface(NetworkInterface* iface)
{
    if(iface != NULL)
    {
        cout << "config: setting interface to " << iface->getName() << endl;
    }
    interface = iface;
}


NetworkInterface* Config::getInterface()
{
    return interface;
}


NetworkInterface* Config::findInterface()
{
    NetworkInterface *ifn;
    ifn = new NetworkInterface();

    // Pull interface names out of /proc/net/wireless
    ifstream *in;
    in = new ifstream("/proc/net/wireless");

    char line[128];
    string *s;
    string *nss;
    while (in->getline(line,127))
    {
        s = new string(line);
        string::size_type colonpos;
        colonpos = s->find_first_of(':');
        // 4294967295 -> this is size_type's idea of zero?
        if(colonpos < 100000000 )
        {
            // We've got an interface
            string::size_type nospacepos;
            nospacepos = s->find_first_not_of(' ');
            // I dont know how to assign this to s, a string*, 
	    // so i went with nss, a string
            nss = new string(s->substr(nospacepos, colonpos-2));
            ifn->setName(nss);
	    cout << "findInterface returning " << *nss << endl;
            return ifn;
        }
        delete s;
    }
    delete in;

    return NULL;

}


void Config::setAPs(AccessPointList* apl)
{
    aplist = apl;
}


AccessPointList* Config::getAPs()
{
    return aplist;
}


void Config::syncAPList(AccessPointList* apl)
{
    // mark all the APs as 'gone away'
    // the mark any APs in apl as 'here'

    // Compare the incoming list with the existing AP List
    cout << "Syncing old APList size " << aplist->size() << " addr:" << aplist << " with new AP list size " << apl->size() << " addr:" << apl << endl;
    AccessPointList::const_iterator newit;
    newit = apl->begin();
    for(;newit != apl->end(); ++newit)
    {
        AccessPointList::const_iterator oldit;
        oldit = aplist->begin();
        cout << "oldit aplist->begin() " << endl;
        bool addme = true;
        for(;oldit != aplist->end(); ++oldit)
        {
            if( (**oldit) == (**newit) )
            {
                // Already have this AP
                cout << "Already have AP bssid: " << (*newit)->getBssidString() << endl;
                addme = false;

		// Check for updates
		if( (*oldit)->hasDifferences(*newit) == true) {
			cout << "the AP has updated values" << endl;
		}
            }
        }
        if(addme == true)
        {
            cout << "Adding AP Essid:" << (*newit)->getEssid() << " addr:" << (*newit) << " " << endl;
            //win->add_AP(*newit);
            aplist->push_back((*newit));
        }
    }
    cout << "SyncAPs finish.  aplist->size() " << aplist->size() << endl;
}


void Config::setPhantomAPs(bool pap)
{
    PhantomAPs = pap;
}


bool Config::getPhantomAPs()
{
    return PhantomAPs;
}


void Config::setDirname(string* dn)
{
    if(dn != 0)
    {
        if(dn->length() > 0)
        {
            dirname = dn;
            //cout << "Config::setDirname setting " << *dn << endl;
        }
        else
        {
            cout << "Config::setDirname given 0 length string" << endl;
        }
    }
    else
    {
        cout << "Config::setDirname given 0 pointer" << endl;
    }
}


string* Config::getDirname()
{
    //cout << "Config::getDirname HERE returning " << " addr:" << dirname << endl;
    //if(dirname != NULL) {
    //	cout << "value: " << *dirname << endl;
    //}
    return dirname;
}


void Config::setDhcpCommand(string* dc)
{
    dhcpCommand = dc;
}


string* Config::getDhcpCommand()
{
    return dhcpCommand;
}


void Config::findResourceDirectory(char* exepath) {

    // Find the resource directory
    // Try the directory where the executable is kept
    struct stat statstruct;
    if(stat(exepath,  &statstruct) == 0)
    {
        // it exists. use it.
        setDirname(new string(exepath));
    }
    else
    {
        // no game, use a default.
        setDirname(new string("/usr/share/apradar"));
    }

    cout << "Resource directory is set to " << *getDirname() << endl;
}

