/* AP Radar
 * Started June 2002
 * donald.g.park@iname.com
 *
 * Released under the MIT Open Source Licence
 * http://www.opensource.org/licenses/mit-license.php
 */

using namespace std;

// C++ header files
#include <string>
#include <iostream>

// C header files
#include <libgen.h>                               // for dirname()
#include <unistd.h>                               // for geteuid()
#include <pthread.h>                              // for pthread_create()
#include <sys/types.h>                            // for socket()
#include <sys/socket.h>                           // for socket()
#include <netinet/in.h>                           // IP constants

// Local header files
#include "Config.h"
#include "AccessPointList.h"
#include "Main.h"
#include "NetworkInterface.h"

// Upon tcpip connection, give the new thread config and socket
struct new_client_data {
    Config* config;
    int sock;
};

// Periodically scans for access points
void* watch(void* c) {

    Config* config = (Config*)c;

    while(1) {
        AccessPointList* apl;
        if(config->getInterface() != NULL) {
            // scan the current environment
            apl = config->getInterface()->scanAPs();
            config->syncAPList(apl);
        }

        sleep(2);
    }
}

void* UIclient(void* ncdp) {
	new_client_data* ncd = (new_client_data*) ncdp;

	int sock = ncd->sock;
	cout << "New client connection established. client socket " << sock << endl;

	string msg;
	msg =+ "You Suck";
	msg =+ sock;
	send(sock, (const void*)msg.c_str(), msg.length(), 0);
	send(sock, (const void*)"helloooo", 9, 0);


}

int main( int   argc,
char *argv[] ) {

    cout << "AP Radar" << VERSION << endl;
    // Start the configuration object
    Config config;

    if(geteuid() != 0) {
        cout << " *** AP Radar daemon should be run as root ***" << endl;
    }

    config.findResourceDirectory(argv[0]);

    NetworkInterface* nif = NULL;

    // command line processing
    if(argc > 1) {
        for (int i=0; i < argc; i++) {
            //cout << "#" << i << " " << argv[i] << endl;
            if(strcmp(argv[i], "-s") == 0) {
                config.setPhantomAPs(true);
            }
            if(strcmp(argv[i], "-i") == 0) {
                nif = new NetworkInterface;
                cout << "i is " << i << argv[i] << endl;
                i++;
                if (i >= argc) {
                    cout << "Specify an interface name with -i. current i = "<< i << endl;
                    return 1;
                }
                nif->setName(new string(argv[i]));
            }
            if(strcmp(argv[i], "-d") == 0) {
                i++;
                if (i >= argc) {
                    cout << "Specify a dhcp client command with -d " << endl;
                }
                config.setDhcpCommand(new string(argv[i]));
                cout << "custom dhcp client command set to \"" << argv[i] << "\"" << endl;
            }
        }
    }

    // main is a utility class of sorts
    cout << "Starting Main" << endl;
    Main main;
    main.setConfig(&config);
    config.setMain(&main);

    // NULL means no interface was specified on the command line
    cout << "Searching for interfaces" << endl;
    if(nif == NULL) {
        // find the first wireless interface in /proc/net/wireless
        nif = config.findInterface();
    }

    // if still null report an error
    if(nif == NULL) {
        cout << "WARNING No wireless interfaces found in /proc/net/wireless" << endl;
        //return 1;
    }
    else {
        nif->setConfig(&config);
        cout << "found \"" << config.getInterface() << "\"" << endl;
    }

    config.setInterface(nif);

    // Setup a new AP list
    cout << "Initializing Access Point List"<< endl;
    config.setAPs(new AccessPointList);

    cout << "starting thread" << endl;
    pthread_t thread;
    int retval;
    retval = pthread_create(&thread, NULL, watch, (void*)&config);
    cout << "thread create returned " << retval << endl;

    while(1) {
        int ret;

        //cout << "listening on socket" << endl;
        int sock;
        sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
        if(sock == -1) {
            cout << "error creating socket" << endl;
        }

        sockaddr_in addr;
        addr.sin_family = AF_INET;
        addr.sin_port = htons(1898);
        addr.sin_addr.s_addr = INADDR_ANY;
        socklen_t len = sizeof(sockaddr_in);
        bind(sock,(const sockaddr *) &addr, len);
        if(ret == -1) {
            cout << "error binding to socket" << endl;
        }

        ret = listen(sock, 1);
        if(ret == -1) {
            cout << "error listening to socket" << endl;
        }

        int clientsock;
        clientsock = accept(sock, (sockaddr *)&addr, &len );
        cout << "CONNECTION!" << endl;

        new_client_data ncd;
        ncd.config = &config;
        ncd.sock = clientsock;

        pthread_t UIthread;
        int retval;
        retval = pthread_create(&UIthread, NULL, UIclient, (void*)&ncd);

        sleep(1);
    }

    return(0);
}


/*
 * Copyright (c) 2002 Donald G. Park
 * The MIT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
