/* AP Radar
 * June 2002
 * donald.g.park@iname.com
 *
 * This method comes from iwlist.c in the wireless-tools package
 *
 * Released under the MIT Open Source Licence
 * http://www.opensource.org/licenses/mit-license.php
 */

#include <iostream>
#include <string>

#include "iwlib.h"
#include "AccessPoint.h"
#include "AccessPointList.h"

AccessPointList*
print_scanning_info(int     skfd,
const char *    ifname_const
)
{
    AccessPointList* aplist;
    aplist = new AccessPointList;

    // AP Radar will compile under wireless extensions of 13 or
    // less, but it wont be able to scan for APs
    #if WIRELESS_EXT > 13
    struct iwreq      wrq;
    unsigned char     buffer[IW_SCAN_MAX_DATA];   /* Results */
    struct timeval    tv;                         /* Select timeout */
    int           timeout = 5000000;              /* 5s */
    char* ifname = (char*)ifname_const;

    /* Init timeout value -> 250ms*/
    tv.tv_sec = 0;
    tv.tv_usec = 250000;

    /*
     * Here we should look at the command line args and set the IW_SCAN_ flags
     * properly
     */
    wrq.u.param.flags = IW_SCAN_DEFAULT;
    wrq.u.param.value = 0;                        /* Later */

    /* Initiate Scanning */
    if(iw_set_ext(skfd, ifname, SIOCSIWSCAN, &wrq) < 0)
    {
        if(errno != EPERM)
        {
            fprintf(stderr, "%-8.8s  Interface doesn't support scanning : %s\n\n",
                ifname, strerror(errno));
            return(0);
        }
        /* If we don't have the permission to initiate the scan, we may
         * still have permission to read left-over results.
         * But, don't wait !!! */
        tv.tv_usec = 0;
    }

    timeout -= tv.tv_usec;

    /* Forever */
    while(1)
    {
        fd_set        rfds;                       /* File descriptors for select */
        int       last_fd;                        /* Last fd */
        int       ret;

        /* Guess what ? We must re-generate rfds each time */
        FD_ZERO(&rfds);
        last_fd = -1;

        /* In here, add the rtnetlink fd in the list */

        /* Wait until something happens */
        ret = select(last_fd + 1, &rfds, NULL, NULL, &tv);

        /* Check if there was an error */
        if(ret < 0)
        {
            if(errno == EAGAIN || errno == EINTR)
                continue;
            fprintf(stderr, "Unhandled signal - exiting...\n");
            return(0);
        }

        /* Check if there was a timeout */
        if(ret == 0)
        {
            /* Try to read the results */
            wrq.u.data.pointer = (char*)buffer;
            wrq.u.data.flags = 0;
            wrq.u.data.length = sizeof(buffer);
            if(iw_get_ext(skfd, ifname, SIOCGIWSCAN, &wrq) < 0)
            {
                /* Check if results not available yet */
                if(errno == EAGAIN)
                {
                    /* Restart timer for only 100ms*/
                    tv.tv_sec = 0;
                    tv.tv_usec = 100000;
                    timeout -= tv.tv_usec;
                    if(timeout > 0)
                        continue;                 /* Try again later */
                }

                /* Bad error */
                fprintf(stderr, "%-8.8s  Failed to read scan data : %s\n\n",
                    ifname, strerror(errno));
                return(aplist);
            }
            else
                /* We have the results, go to process them */
                break;
        }

        /* In here, check if event and event type
         * if scan event, read results. All errors bad & no reset timeout */
    }

    if(wrq.u.data.length)
    {
        struct iw_event       iwe;
        struct stream_descr   stream;
        int           ap_num = 1;
        int           ret;
        struct iw_range       range;
        int           has_range;

        has_range = (iw_get_range_info(skfd, ifname, &range) >= 0);
        printf("%-8.8s  Scan completed :\n", ifname);
        //iw_init_event_stream(&stream, (char*)buffer, wrq.u.data.length);
        memset((char *) &stream, '\0', sizeof(struct stream_descr));
        /* Set things up */
        stream.current = (char *)buffer;
        stream.end = (char *)buffer + wrq.u.data.length;

        // loop through all the AP information
        // how do I associate an event with an AP?
        do
        {
            /* Extract an event and print it */
            ret = iw_extract_event_stream(&stream, &iwe);
            if(ret > 0)
            {
                /* Now, let's decode the event */
                switch(iwe.cmd)
                {

                    case SIOCGIWAP:
                        // Allocate a new AP structure
                        AccessPoint* newap;
                        newap = new AccessPoint;
                        aplist->push_back(newap);
                        cout << "                    NEW AP event" << endl;
                        newap->setBssid(iwe.u.ap_addr.sa_data);
                        break;

                    case SIOCGIWMODE:
                        printf("                    Mode:%s %d\n",
                            iw_operation_mode[iwe.u.mode], iwe.u.mode);
                        // int32 mode. 1 = ad-hoc
                        newap->setMode(iwe.u.mode);
                        break;

                    case SIOCGIWESSID:
                        char essid[IW_ESSID_MAX_SIZE+1];
                        if((iwe.u.essid.pointer) && (iwe.u.essid.length))
                            memcpy(essid, iwe.u.essid.pointer, iwe.u.essid.length);
                        essid[iwe.u.essid.length] = '\0';
                        if(iwe.u.essid.flags)
                        {
                            /* Does it have an ESSID index ? */
                            if((iwe.u.essid.flags & IW_ENCODE_INDEX) > 1)
                                printf("                    ESSID:\"%s\" [%d]\n", essid,
                                    (iwe.u.essid.flags & IW_ENCODE_INDEX));
                            else
                            {
                                printf("                    ESSID:\"%s\"\n", essid);
                                //awin->add_button(essid);
                                newap->setEssid(new string(essid));
                            }
                        }
                        else
                            printf("                    ESSID:off/any\n");
                        break;

                    case SIOCGIWENCODE:
                        unsigned char   key[IW_ENCODING_TOKEN_MAX];
                        if(iwe.u.data.pointer)
                            memcpy(key, iwe.u.essid.pointer, iwe.u.data.length);
                        else
                            iwe.u.data.flags |= IW_ENCODE_NOKEY;
                        printf("                    Encryption key:");
                        if(iwe.u.data.flags & IW_ENCODE_DISABLED)
                        {
                            printf("off\n");
                            newap->setWEP(0);
                        }
                        else
                        {
                            newap->setWEP(1);
                        }
                        break;

                    case SIOCGIWFREQ:
                        float           freq;     /* Frequency/channel */
                        char buffer[128];
                        freq = (double)iwe.u.freq.m * pow(10, (double)iwe.u.freq.e);
                        if(freq < KILO)
                            sprintf(buffer, "Channel:%g", freq);
                        else
                        {
                            if(freq >= GIGA)
                                sprintf(buffer, "Frequency:%gGHz", freq / GIGA);
                            else
                            {
                                if(freq >= MEGA)
                                    sprintf(buffer, "Frequency:%gMHz", freq / MEGA);
                                else
                                    sprintf(buffer, "Frequency:%gkHz", freq / KILO);
                            }
                        }

                        printf("                    %s\n", buffer);
                        newap->setFrequency(freq);
                        break;

                    case SIOCGIWRATE:
                        //double rate = iwe.u.bitrate.value;
                        printf("                    Bit Rate:%d\n", iwe.u.bitrate.value);

                        break;

                    case IWEVQUAL:
                        /* Not that reliable, disable */
                        iwe.u.qual.updated = 0x0;

                        if(has_range && (iwe.u.qual.level != 0))
                        {
                            /* If the statistics are in dBm */
                            if(iwe.u.qual.level > range.max_qual.level)
                            {
                                /* Statistics are in dBm (absolute power measurement) */
				 int quality = iwe.u.qual.level - 0x100;
                                printf("             quality: %d\n", quality);
				newap->setSignalStrength(quality);

                            }
                        }

                        break;

                        // ap_num = print_scanning_token(&iwe, ap_num, &range, has_range);
                }
            }
            else
                printf("%-8.8s  No scan results\n", ifname);
        }
        while(ret > 0);
        printf("\n");

        #else
        #warning Kernel version doesnt support wireless scanning
        printf("AP Radar compiled with kernel scanning capabilites (pre-2.4.20).\n");
        #endif

        return(aplist);
    }

}


// AP Radar will compile under wireless extensions of 13 or
// less, but it wont be able to scan for APs
#if WIRELESS_EXT > 13

// From iwlib.c
//

/* Type of headers we know about (basically union iwreq_data) */
#define IW_HEADER_TYPE_NULL 0                     /* Not available */
#define IW_HEADER_TYPE_CHAR 2                     /* char [IFNAMSIZ] */
#define IW_HEADER_TYPE_UINT 4                     /* __u32 */
#define IW_HEADER_TYPE_FREQ 5                     /* struct iw_freq */
#define IW_HEADER_TYPE_POINT    6                 /* struct iw_point */
#define IW_HEADER_TYPE_PARAM    7                 /* struct iw_param */
#define IW_HEADER_TYPE_ADDR 8                     /* struct sockaddr */
#define IW_HEADER_TYPE_QUAL 9                     /* struct iw_quality */

/* Headers for the various requests */
static const char standard_ioctl_hdr[] =
{
    IW_HEADER_TYPE_NULL,                          /* SIOCSIWCOMMIT */
    IW_HEADER_TYPE_CHAR,                          /* SIOCGIWNAME */
    IW_HEADER_TYPE_PARAM,                         /* SIOCSIWNWID */
    IW_HEADER_TYPE_PARAM,                         /* SIOCGIWNWID */
    IW_HEADER_TYPE_FREQ,                          /* SIOCSIWFREQ */
    IW_HEADER_TYPE_FREQ,                          /* SIOCGIWFREQ */
    IW_HEADER_TYPE_UINT,                          /* SIOCSIWMODE */
    IW_HEADER_TYPE_UINT,                          /* SIOCGIWMODE */
    IW_HEADER_TYPE_PARAM,                         /* SIOCSIWSENS */
    IW_HEADER_TYPE_PARAM,                         /* SIOCGIWSENS */
    IW_HEADER_TYPE_NULL,                          /* SIOCSIWRANGE */
    IW_HEADER_TYPE_POINT,                         /* SIOCGIWRANGE */
    IW_HEADER_TYPE_NULL,                          /* SIOCSIWPRIV */
    IW_HEADER_TYPE_POINT,                         /* SIOCGIWPRIV */
    IW_HEADER_TYPE_NULL,                          /* SIOCSIWSTATS */
    IW_HEADER_TYPE_POINT,                         /* SIOCGIWSTATS */
    IW_HEADER_TYPE_POINT,                         /* SIOCSIWSPY */
    IW_HEADER_TYPE_POINT,                         /* SIOCGIWSPY */
    IW_HEADER_TYPE_NULL,                          /* -- hole -- */
    IW_HEADER_TYPE_NULL,                          /* -- hole -- */
    IW_HEADER_TYPE_ADDR,                          /* SIOCSIWAP */
    IW_HEADER_TYPE_ADDR,                          /* SIOCGIWAP */
    IW_HEADER_TYPE_NULL,                          /* -- hole -- */
    IW_HEADER_TYPE_POINT,                         /* SIOCGIWAPLIST */
    IW_HEADER_TYPE_PARAM,                         /* SIOCSIWSCAN */
    IW_HEADER_TYPE_POINT,                         /* SIOCGIWSCAN */
    IW_HEADER_TYPE_POINT,                         /* SIOCSIWESSID */
    IW_HEADER_TYPE_POINT,                         /* SIOCGIWESSID */
    IW_HEADER_TYPE_POINT,                         /* SIOCSIWNICKN */
    IW_HEADER_TYPE_POINT,                         /* SIOCGIWNICKN */
    IW_HEADER_TYPE_NULL,                          /* -- hole -- */
    IW_HEADER_TYPE_NULL,                          /* -- hole -- */
    IW_HEADER_TYPE_PARAM,                         /* SIOCSIWRATE */
    IW_HEADER_TYPE_PARAM,                         /* SIOCGIWRATE */
    IW_HEADER_TYPE_PARAM,                         /* SIOCSIWRTS */
    IW_HEADER_TYPE_PARAM,                         /* SIOCGIWRTS */
    IW_HEADER_TYPE_PARAM,                         /* SIOCSIWFRAG */
    IW_HEADER_TYPE_PARAM,                         /* SIOCGIWFRAG */
    IW_HEADER_TYPE_PARAM,                         /* SIOCSIWTXPOW */
    IW_HEADER_TYPE_PARAM,                         /* SIOCGIWTXPOW */
    IW_HEADER_TYPE_PARAM,                         /* SIOCSIWRETRY */
    IW_HEADER_TYPE_PARAM,                         /* SIOCGIWRETRY */
    IW_HEADER_TYPE_POINT,                         /* SIOCSIWENCODE */
    IW_HEADER_TYPE_POINT,                         /* SIOCGIWENCODE */
    IW_HEADER_TYPE_PARAM,                         /* SIOCSIWPOWER */
    IW_HEADER_TYPE_PARAM,                         /* SIOCGIWPOWER */
};
static const unsigned int standard_ioctl_num = sizeof(standard_ioctl_hdr);

/*
 * Meta-data about all the additional standard Wireless Extension events
 * we know about.
 */
static const char   standard_event_hdr[] =
{
    IW_HEADER_TYPE_ADDR,                          /* IWEVTXDROP */
    IW_HEADER_TYPE_QUAL,                          /* IWEVQUAL */
};
static const unsigned int standard_event_num = sizeof(standard_event_hdr);

/* Size (in bytes) of various events */
static const int event_type_size[] =
{
    IW_EV_LCP_LEN,
    0,
    IW_EV_CHAR_LEN,
    0,
    IW_EV_UINT_LEN,
    IW_EV_FREQ_LEN,
    IW_EV_POINT_LEN,                              /* Without variable payload */
    IW_EV_PARAM_LEN,
    IW_EV_ADDR_LEN,
    IW_EV_QUAL_LEN,
};

/*------------------------------------------------------------------*/
/*
 * Get the range information out of the driver
 */
int
iw_get_range_info(int       skfd,
char *    ifname,
iwrange * range)
{
    struct iwreq      wrq;
    char          buffer[sizeof(iwrange) * 2];    /* Large enough */

    /* Cleanup */
    memset(buffer, 0, sizeof(buffer));

    wrq.u.data.pointer = (caddr_t) buffer;
    wrq.u.data.length = sizeof(buffer);
    wrq.u.data.flags = 0;
    if(iw_get_ext(skfd, ifname, SIOCGIWRANGE, &wrq) < 0)
        return(-1);

    /* Copy stuff at the right place, ignore extra */
    memcpy((char *) range, buffer, sizeof(iwrange));

    /* Lots of people have driver and tools out of sync as far as Wireless
     * Extensions are concerned. It's because /usr/include/linux/wireless.h
     * and /usr/src/linux/include/linux/wireless.h are different.
     * We try to catch this stuff here... */

    /* For new versions, we can check the version directly, for old versions
     * we use magic. 300 bytes is a also magic number, don't touch... */
    if((WIRELESS_EXT > 10) && (wrq.u.data.length >= 300))
    {
        #if WIRELESS_EXT > 10
        /* Version verification - for new versions */
        if(range->we_version_compiled != WIRELESS_EXT)
        {
            fprintf(stderr, "Warning : Device %s has been compiled with version %d\n", ifname, range->we_version_compiled);
            fprintf(stderr, "of Wireless Extension, while we are using version %d.\n", WIRELESS_EXT);
            fprintf(stderr, "Some things may be broken...\n\n");
        }
        #endif                                    /* WIRELESS_EXT > 10 */
    }
    else
    {
        /* Version verification - for old versions */
        if(wrq.u.data.length != sizeof(iwrange))
        {
            fprintf(stderr, "Warning : Device %s has been compiled with a different version\n", ifname);
            fprintf(stderr, "of Wireless Extension than ours (we are using version %d).\n", WIRELESS_EXT);
            fprintf(stderr, "Some things may be broken...\n\n");
        }
    }

    /* Note : we are only trying to catch compile difference, not source.
     * If the driver source has not been updated to the latest, it doesn't
     * matter because the new fields are set to zero */

    return(0);
}


/*------------------------------------------------------------------*/
/*
 * Extract the next event from the event stream.
 */
int
                                                  /* Stream of events */
iw_extract_event_stream(struct stream_descr *   stream,
struct iw_event *   iwe)                          /* Extracted event */
{
    int       event_type = 0;
    int       event_len = 1;                      /* Invalid */
    char *    pointer;
    /* Don't "optimise" the following variable, it will crash */
    unsigned  cmd_index;                          /* *MUST* be unsigned */

    /* Check for end of stream */
    if((stream->current + IW_EV_LCP_LEN) > stream->end)
        return(0);

    #if 0
    printf("DBG - stream->current = %p, stream->value = %p, stream->end = %p\n",
        stream->current, stream->value, stream->end);
    #endif

    /* Extract the event header (to get the event id).
     * Note : the event may be unaligned, therefore copy... */
    memcpy((char *) iwe, stream->current, IW_EV_LCP_LEN);

    #if 0
    printf("DBG - iwe->cmd = 0x%X, iwe->len = %d\n",
        iwe->cmd, iwe->len);
    #endif

    /* Get the type and length of that event */
    if(iwe->cmd <= SIOCIWLAST)
    {
        cmd_index = iwe->cmd - SIOCIWFIRST;
        if(cmd_index < standard_ioctl_num)
            event_type = standard_ioctl_hdr[cmd_index];
    }
    else
    {
        cmd_index = iwe->cmd - IWEVFIRST;
        if(cmd_index < standard_event_num)
            event_type = standard_event_hdr[cmd_index];
    }
    event_len = event_type_size[event_type];

    /* Check if we know about this event */
    if((event_len == 0) || (iwe->len == 0))
        return(-1);
    event_len -= IW_EV_LCP_LEN;

    /* Set pointer on data */
    if(stream->value != NULL)
        pointer = stream->value;                  /* Next value in event */
    else
                                                  /* First value in event */
        pointer = stream->current + IW_EV_LCP_LEN;

    #if 0
    printf("DBG - event_type = %d, event_len = %d, pointer = %p\n",
        event_type, event_len, pointer);
    #endif

    /* Copy the rest of the event (at least, fixed part) */
    if((pointer + event_len) > stream->end)
        return(-2);
    memcpy((char *) iwe + IW_EV_LCP_LEN, pointer, event_len);

    /* Skip event in the stream */
    pointer += event_len;

    /* Special processing for iw_point events */
    if(event_type == IW_HEADER_TYPE_POINT)
    {
        /* Check the length of the payload */
        if((iwe->len - (event_len + IW_EV_LCP_LEN)) > 0)
            /* Set pointer on variable part (warning : non aligned) */
            iwe->u.data.pointer = pointer;
        else
            /* No data */
            iwe->u.data.pointer = NULL;

        /* Go to next event */
        stream->current += iwe->len;
    }
    else
    {
        /* Is there more value in the event ? */
        if((pointer + event_len) <= (stream->current + iwe->len))
            /* Go to next value */
            stream->value = pointer;
        else
        {
            /* Go to next event */
            stream->value = NULL;
            stream->current += iwe->len;
        }
    }
    return(1);
}
#endif

/*
 * Copyright (c) 2002 Donald G. Park
 * The MIT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
