/* AP Radar
 * June 2002
 * donald.g.park@iname.com
 *
 * Released under the MIT Open Source Licence
 * http://www.opensource.org/licenses/mit-license.php
 */

#include <string>
#include <iostream>
#include <sstream>
#include <arpa/inet.h>
#include <netinet/ip_icmp.h>
#include "iwlib.h"
#include "NetworkInterface.h"
#include "AccessPoint.h"
#include "Config.h"

// The constructor
//NetworkInterface::NetworkInterface() {
//}

// The destructor
//NetworkInterface::~NetworkInterface() {
//}

void NetworkInterface::setName(string* n)
{
    name = n;
}


string* NetworkInterface::getName()
{
    return name;
}


void NetworkInterface::Associate(AccessPoint* ap)
{
    // Set this interface to the parameters specified in the passed-in AccessPoint
    setEssid(ap->getEssid());
    setMode(ap->getMode());

    // Kick off a DHCP request
    ostringstream o;
    o << *(getConfig()->getDhcpCommand()) << " " << *getName() << " & ";
    string* cmdline = new string(o.str());
    cout << "Launching \"" << *cmdline << "\"" << endl;
    system(cmdline->c_str());

    // pull the Gateway out of /etc/dhcpcd/dhcpcd-ifname.info

}


void NetworkInterface::setEssid(string* e)
{
    cout << "Setting Essid " << *e << " on " << *name << endl;
    //  IW_SET_EXT_ERR(skfd, ifname, SIOCSIWESSID, &wrq, "Set ESSID");
    struct iwreq wrq;
    wrq.u.essid.flags = 1;
    wrq.u.essid.pointer = (caddr_t) e->c_str();
    wrq.u.essid.length = strlen(e->c_str()) + 1;
    if(iw_set_ext(getConfig()->getSkfd(), (char*)getName()->c_str(), SIOCSIWESSID, &wrq) < 0)
    {
        cout << "Error setting essid" << endl;
    }

}


string* NetworkInterface::getEssid()
{
    //	return essid;
    /* Get ESSID */
    struct iwreq wrq;
    char str[64];
    wrq.u.essid.pointer = (caddr_t) str;
    wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
    wrq.u.essid.flags = 0;
    if(iw_get_ext(getConfig()->getSkfd(), (char*)name->c_str(), SIOCGIWESSID, &wrq) >= 0)
    {
        printf("getEssid \"%s\" (%d)\n", str, strlen(str));
        return new string(str);
    }

    return new string("unknown");
}


void NetworkInterface::setBssid(char* b)
{
}


char* NetworkInterface::getBssid()
{
    return "";
}


void NetworkInterface::setMode(int m)
{
    struct iwreq wrq;
    //IW_SET_EXT_ERR(skfd, ifname, SIOCSIWMODE, &wrq, "Set Mode");
    wrq.u.mode = m;
    if(iw_set_ext(getConfig()->getSkfd(), (char*)getName()->c_str(), SIOCSIWMODE, &wrq) < 0)
    {
        cout << "Error setting operation mode" << endl;
    }

}


int NetworkInterface::getMode()
{

    struct iwreq wrq;
    /* Get operation mode */
    if(iw_get_ext(getConfig()->getSkfd(), (char*)name->c_str(), SIOCGIWMODE, &wrq) >= 0)
    {
        return wrq.u.mode;
    }

    return -1;
}


string* NetworkInterface::getModeString()
{
    int mode = getMode();
    if(mode == -1)
        return new string("unknown");
    if(mode == 1)
        return new string("ad-hoc");
    if(mode == 2)
        return new string("managed");
}


void NetworkInterface::setWEP(int w)
{
}


int NetworkInterface::getWEP()
{
    return 0;
}


void NetworkInterface::setFrequency(float f)
{
}


float NetworkInterface::getFrequency()
{
    return 0;
}

string* NetworkInterface::getDriverName() {
	struct stat buf;

	// test for aironet
	// look for /proc/driver/aironet
	//const char *filename = "/proc/driver/aironet/" + getName()->c_str();
	ostringstream o;
	o << "/proc/driver/aironet/";
	o << *(getName());
	// more weird c++ shit
	string oo = o.str(); // convert on ostringstream to a string
	cout << "testing " << oo << endl;
	int retval = stat(oo.c_str() , &buf); // string into char* buffer and stat it
	if(retval == 0) {
		if(S_ISDIR(buf.st_mode)) {
			return new string("aironet");
		}
	}
	
	// test for hostap

	return new string("unknown");
}

void NetworkInterface::getStats()
{

    // iwstats *  stats;

    // struct iwreq          wrq;
    //   wrq.u.data.pointer = (caddr_t) stats;
    //    wrq.u.data.length = 0;
    //     wrq.u.data.flags = 1;         /* Clear updated flag */
    //  strncpy(wrq.ifr_name, ifname, IFNAMSIZ);
    //   if(iw_get_ext(skfd, ifname, SIOCGIWSTATS, &wrq) < 0)
    //  return false;
}


void NetworkInterface::ping(string* host)
{
    // ping the first default gateway that uses this interface
    int icmp_sock;                                /* socket file descriptor */
    icmp_sock = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP);
    if(icmp_sock == -1)
    {
        fprintf(stdout,"ping socket creation error. not running as root?\n");
        return;
    }

    // some borrowed lines from libping
    struct hostent  *hp = NULL;
    //struct protoent *proto = NULL;

    //proto  = getprotobyname( "icmp" ); 
    hp     = gethostbyname( host->c_str() );

    struct sockaddr_in taddr;
    if(hp != NULL ){
    memcpy( &taddr.sin_addr, hp->h_addr_list[0], sizeof( taddr.sin_addr ));
    taddr.sin_port = 0;
    taddr.sin_family = AF_INET;
    } else if( inet_aton( host->c_str(), &taddr.sin_addr ) == 0 ){
        return;
    }

#define ICMP_MINLEN             8
#define HDRLEN      ICMP_MINLEN
#define PKTSIZE     64 
#define DATALEN     (PKTSIZE-HDRLEN)
    unsigned char buf[ HDRLEN + DATALEN ];
    struct icmp     *icp;
    icp = (struct icmp *)buf;
    icp->icmp_type  = ICMP_ECHO;
    icp->icmp_code  = 0;
    icp->icmp_cksum = 0;
    icp->icmp_id    = getpid() & 0xFFFF;
    icp->icmp_cksum = ipchecksum((unsigned short *)icp, HDRLEN+DATALEN );

    int retval;
    const sockaddr* fyouaddr = (const sockaddr*)&taddr;
    if(( retval = sendto( icmp_sock, buf, sizeof( buf ), 0, 
      fyouaddr, sizeof( taddr ))) < 0 ){
        cout << "ping sendto error" << endl;
        return ;
    }


}


Ipv4Address* NetworkInterface::getIpv4Address()
{
    struct ifreq ifr;
    struct sockaddr_in *sin = (struct sockaddr_in *)&ifr.ifr_addr;
    int sockfd, i;

    bzero(&ifr, sizeof(ifr));

    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if(getName() != NULL)
    {
        strcpy(ifr.ifr_name, getName()->c_str());
    }
    else
    {
        return NULL;
        cout << "cant get IP address. no interface name. " <<  endl;
    }
    sin->sin_family = AF_INET;

    if(ioctl(sockfd, SIOCGIFADDR, &ifr) == 0)
    {
        unsigned int addr = sin->sin_addr.s_addr;
        return new Ipv4Address( addr & 0xff,
            (addr & 0xff00) >> 8,
            (addr & 0xff0000) >> 16,
            (addr & 0xff000000) >> 24);
    }

    cout << "getting IP for " << *getName()  << " failed." << endl;
    return NULL;
}


unsigned short NetworkInterface::ipchecksum(unsigned short *addr, register int len)
{

    register int nleft = len;
    const u_short *w = addr;
    register u_short answer;
    register int sum = 0;

    /*
     *  Our algorithm is simple, using a 32 bit accumulator (sum),
     *  we add sequential 16 bit words to it, and at the end, fold
     *  back all the carry bits from the top 16 bits into the lower
     *  16 bits.
     */
    while (nleft > 1)
    {
        sum += *w++;
        nleft -= 2;
    }

    /* mop up an odd byte, if necessary */
    if (nleft == 1)
        sum += htons(*(u_char *)w << 8);

    /*
     * add back carry outs from top 16 bits to low 16 bits
     */
    sum = (sum >> 16) + (sum & 0xffff);           /* add hi 16 to low 16 */
    sum += (sum >> 16);                           /* add carry */
    answer = ~sum;                                /* truncate to 16 bits */
    return (answer);
}

string* NetworkInterface::findDefaultRoute() {

	// the lame way - open /proc/net/route
	

	// the more kernel-foo way, open a NETLINK socket

}


/*
 * Copyright (c) 2002 Donald G. Park
 * The MT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
